"""
Created on 22 Mar 2014

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2014 Ronny Andersson
@license: MIT
"""

__all__ = [
    'TAPS',
    ]

# All taps are found on this website:
# www.newwaveinstruments.com/resources/articles/m_sequence_linear_feedback_shift_register_lfsr.htm
#
# Taps are always 1-indexed, i.e. they start at 1 (one). So index 1 means
# first (rightmost) tap.
TAPS = {
    3 : (
        (3, 2),
        ),
    4 : (
        (4, 3),
        ),
    5 : (
        (5, 3),
        (5, 4, 3, 2),
        (5, 4, 3, 1),
        ),
    6 : (
        (6, 5),
        (6, 5, 4, 1),
        (6, 5, 3, 2),
        ),
    7 : (
        (7, 6),
        (7, 4),
        (7, 6, 5, 4),
        (7, 6, 5, 2),
        (7, 6, 4, 2),
        (7, 6, 4, 1),
        (7, 5, 4, 3),
        (7, 6, 5, 4, 3, 2),
        (7, 6, 5, 4, 2, 1),
        ),
    8 : (
        (8, 7, 6, 1),
        (8, 7, 5, 3),
        (8, 7, 3, 2),
        (8, 6, 5, 4),
        (8, 6, 5, 3),
        (8, 6, 5, 2),
        (8, 7, 6, 5, 4, 2),
        (8, 7, 6, 5, 2, 1),
        ),
    9 : (
        (9, 5),
        (9, 8, 7, 2),
        (9, 8, 6, 5),
        (9, 8, 5, 4),
        (9, 8, 5, 1),
        (9, 8, 4, 2),
        (9, 7, 6, 4),
        (9, 7, 5, 2),
        (9, 6, 5, 3),
        (9, 8, 7, 6, 5, 3),
        (9, 8, 7, 6, 5, 1),
        (9, 8, 7, 6, 4, 3),
        (9, 8, 7, 6, 4, 2),
        (9, 8, 7, 6, 3, 2),
        (9, 8, 7, 6, 3, 1),
        (9, 8, 7, 6, 2, 1),
        (9, 8, 7, 5, 4, 3),
        (9, 8, 7, 5, 4, 2),
        (9, 8, 6, 5, 4, 1),
        (9, 8, 6, 5, 3, 2),
        (9, 8, 6, 5, 3, 1),
        (9, 7, 6, 5, 4, 3),
        (9, 7, 6, 5, 4, 2),
        (9, 8, 7, 6, 5, 4, 3, 1),
        ),
    10: (
        (10, 7),
        (10, 9, 8, 5),
        (10, 9, 7, 6),
        (10, 9, 7, 3),
        (10, 9, 6, 1),
        (10, 9, 5, 2),
        (10, 9, 4, 2),
        (10, 8, 7, 5),
        (10, 8, 7, 2),
        (10, 8, 5, 4),
        (10, 8, 4, 3),
        (10, 9, 8, 7, 5, 4),
        (10, 9, 8, 7, 4, 1),
        (10, 9, 8, 7, 3, 2),
        (10, 9, 8, 6, 5, 1),
        (10, 9, 8, 6, 4, 3),
        (10, 9, 8, 6, 4, 2),
        (10, 9, 8, 6, 3, 2),
        (10, 9, 8, 6, 2, 1),
        (10, 9, 8, 5, 4, 3),
        (10, 9, 8, 4, 3, 2),
        (10, 9, 7, 6, 4, 1),
        (10, 9, 7, 5, 4, 2),
        (10, 9, 6, 5, 4, 3),
        (10, 8, 7, 6, 5, 2),
        (10, 9, 8, 7, 6, 5, 4, 3),
        (10, 9, 8, 7, 6, 5, 4, 1),
        (10, 9, 8, 7, 6, 4, 3, 1),
        (10, 9, 8, 6, 5, 4, 3, 2),
        (10, 9, 7, 6, 5, 4, 3, 2),
        ),
    11: (
        (11, 9),
        (11, 10, 9, 7),
        (11, 10, 9, 5),
        (11, 10, 9, 2),
        (11, 10, 8, 6),
        (11, 10, 8, 1),
        (11, 10, 7, 3),
        (11, 10, 7, 2),
        (11, 10, 6, 5),
        (11, 10, 4, 3),
        (11, 10, 3, 2),
        (11, 9, 8, 6),
        (11, 9, 8, 4),
        (11, 9, 8, 3),
        (11, 9, 7, 4),
        (11, 9, 7, 2),
        (11, 9, 6, 5),
        (11, 9, 6, 3),
        (11, 9, 5, 3),
        (11, 8, 6, 4),
        (11, 8, 6, 3),
        (11, 7, 6, 5),
        (11, 7, 6, 4),
        (11, 10, 9, 8, 7, 4),
        (11, 10, 9, 8, 7, 1),
        (11, 10, 9, 8, 5, 4),
        (11, 10, 9, 8, 4, 3),
        (11, 10, 9, 8, 3, 1),
        (11, 10, 9, 7, 5, 1),
        (11, 10, 9, 7, 4, 1),
        (11, 10, 9, 6, 5, 4),
        (11, 10, 9, 6, 4, 2),
        (11, 10, 9, 6, 3, 1),
        (11, 10, 9, 6, 2, 1),
        (11, 10, 9, 5, 4, 3),
        (11, 10, 9, 5, 4, 1),
        (11, 10, 9, 5, 3, 1),
        (11, 10, 9, 4, 3, 2),
        (11, 10, 8, 7, 6, 5),
        (11, 10, 8, 7, 6, 3),
        (11, 10, 8, 7, 5, 3),
        (11, 10, 8, 7, 4, 1),
        (11, 10, 8, 6, 5, 4),
        (11, 10, 8, 6, 5, 1),
        (11, 10, 8, 6, 4, 3),
        (11, 10, 8, 6, 4, 2),
        (11, 10, 8, 5, 3, 2),
        (11, 10, 8, 4, 3, 2),
        (11, 10, 7, 6, 5, 3),
        (11, 10, 7, 6, 5, 1),
        (11, 10, 7, 6, 4, 2),
        (11, 10, 7, 6, 4, 1),
        (11, 10, 7, 6, 3, 2),
        (11, 10, 7, 4, 3, 2),
        (11, 9, 8, 7, 6, 3),
        (11, 9, 8, 7, 4, 2),
        (11, 9, 8, 6, 5, 2),
        (11, 9, 8, 6, 4, 3),
        (11, 9, 7, 6, 5, 4),
        (11, 9, 7, 6, 5, 3),
        (11, 9, 7, 6, 4, 2),
        (11, 9, 6, 5, 4, 3),
        (11, 8, 7, 6, 4, 3),
        ),
    12: (
        (12, 11, 10, 4),
        (12, 11, 10, 2),
        (12, 11, 8, 6),
        (12, 11, 7, 4),
        (12, 10, 9, 3),
        (12, 10, 5, 4),
        (12, 9, 8, 5),
        (12, 9, 7, 6),
        (12, 8, 6, 5),
        (12, 11, 10, 9, 8, 4),
        (12, 11, 10, 9, 6, 4),
        (12, 11, 10, 9, 6, 2),
        (12, 11, 10, 9, 4, 3),
        (12, 11, 10, 9, 4, 2),
        (12, 11, 10, 8, 7, 3),
        (12, 11, 10, 8, 7, 2),
        (12, 11, 10, 8, 6, 4),
        (12, 11, 10, 8, 6, 1),
        (12, 11, 10, 8, 5, 1),
        (12, 11, 10, 8, 2, 1),
        (12, 11, 10, 7, 6, 3),
        (12, 11, 10, 7, 6, 2),
        (12, 11, 10, 7, 5, 2),
        (12, 11, 10, 7, 2, 1),
        (12, 11, 10, 6, 3, 2),
        (12, 11, 9, 8, 7, 6),
        (12, 11, 9, 8, 7, 4),
        (12, 11, 9, 8, 4, 1),
        (12, 11, 9, 8, 3, 1),
        (12, 11, 9, 7, 6, 5),
        (12, 11, 9, 7, 6, 4),
        (12, 11, 9, 7, 3, 1),
        (12, 11, 9, 6, 5, 3),
        (12, 11, 9, 5, 4, 1),
        (12, 11, 8, 7, 6, 3),
        (12, 11, 8, 7, 5, 4),
        (12, 11, 8, 6, 5, 2),
        (12, 11, 8, 5, 4, 2),
        (12, 11, 7, 5, 4, 3),
        (12, 11, 6, 4, 3, 2),
        (12, 10, 9, 8, 7, 6),
        (12, 10, 9, 8, 7, 3),
        (12, 10, 9, 8, 6, 2),
        (12, 10, 9, 8, 4, 3),
        (12, 10, 9, 8, 4, 2),
        (12, 10, 9, 7, 5, 2),
        (12, 10, 9, 6, 4, 3),
        (12, 10, 9, 6, 4, 2),
        (12, 10, 8, 7, 6, 2),
        (12, 9, 8, 7, 5, 3),
        (12, 9, 8, 7, 4, 3),
        (12, 9, 8, 6, 5, 4),
        ),
    13: (
        (13, 12, 11, 8),
        (13, 12, 11, 2),
        (13, 12, 11, 1),
        (13, 12, 10, 9),
        (13, 12, 10, 6),
        (13, 12, 10, 3),
        (13, 12, 9, 7),
        (13, 12, 9, 3),
        (13, 12, 8, 2),
        (13, 12, 7, 6),
        (13, 12, 7, 5),
        (13, 12, 7, 4),
        (13, 12, 6, 3),
        (13, 12, 4, 3),
        (13, 12, 4, 2),
        (13, 11, 10, 6),
        (13, 11, 10, 5),
        (13, 11, 9, 8),
        (13, 11, 9, 5),
        (13, 11, 8, 7),
        (13, 11, 8, 6),
        (13, 11, 8, 4),
        (13, 11, 8, 3),
        (13, 11, 7, 2),
        (13, 11, 5, 4),
        (13, 11, 4, 3),
        (13, 10, 9, 4),
        (13, 10, 8, 5),
        (13, 10, 8, 3),
        (13, 10, 7, 6),
        (13, 10, 6, 5),
        (13, 10, 6, 4),
        (13, 8, 7, 6),
        (13, 12, 11, 10, 9, 6),
        (13, 12, 11, 10, 9, 3),
        (13, 12, 11, 10, 9, 2),
        (13, 12, 11, 10, 8, 7),
        (13, 12, 11, 10, 8, 6),
        (13, 12, 11, 10, 8, 5),
        (13, 12, 11, 10, 8, 1),
        (13, 12, 11, 10, 7, 1),
        ),
    14: (
        (14, 13, 12, 2),
        (14, 13, 11, 9),
        (14, 13, 11, 4),
        (14, 13, 10, 8),
        (14, 13, 10, 6),
        (14, 13, 10, 3),
        (14, 13, 8, 6),
        (14, 13, 8, 4),
        (14, 13, 8, 3),
        (14, 13, 7, 3),
        (14, 13, 4, 2),
        (14, 13, 3, 2),
        (14, 12, 11, 6),
        (14, 12, 11, 5),
        (14, 12, 9, 2),
        (14, 12, 7, 5),
        (14, 11, 10, 9),
        (14, 11, 10, 3),
        (14, 11, 9, 7),
        (14, 11, 6, 5),
        (14, 9, 8, 5),
        (14, 13, 12, 11, 10, 8),
        (14, 13, 12, 11, 10, 2),
        (14, 13, 12, 11, 9, 7),
        (14, 13, 12, 11, 9, 1),
        (14, 13, 12, 11, 8, 7),
        (14, 13, 12, 11, 5, 4),
        (14, 13, 12, 11, 5, 2),
        (14, 13, 12, 11, 4, 2),
        (14, 13, 12, 10, 8, 5),
        (14, 13, 12, 10, 4, 2),
        (14, 13, 12, 10, 4, 1),
        (14, 13, 12, 9, 8, 3),
        (14, 13, 12, 9, 8, 1),
        (14, 13, 12, 9, 7, 6),
        (14, 13, 12, 9, 5, 4),
        (14, 13, 12, 8, 7, 5),
        (14, 13, 12, 8, 5, 4),
        (14, 13, 12, 8, 4, 3),
        (14, 13, 12, 8, 4, 2),
        (14, 13, 12, 8, 4, 1),
        ),
    15: (
        (15, 14),
        (15, 11),
        (15, 8),
        (15, 14, 13, 11),
        (15, 14, 13, 8),
        (15, 14, 13, 1),
        (15, 14, 12, 3),
        (15, 14, 12, 2),
        (15, 14, 11, 8),
        (15, 14, 11, 6),
        (15, 14, 11, 1),
        (15, 14, 10, 5),
        (15, 14, 10, 2),
        (15, 14, 9, 8),
        (15, 14, 9, 3),
        (15, 14, 9, 2),
        (15, 14, 8, 5),
        (15, 14, 8, 4),
        (15, 14, 7, 2),
        (15, 14, 6, 3),
        (15, 14, 5, 2),
        (15, 13, 12, 10),
        (15, 13, 11, 10),
        (15, 13, 11, 7),
        (15, 13, 10, 8),
        (15, 13, 10, 7),
        (15, 13, 10, 3),
        (15, 13, 10, 2),
        (15, 13, 9, 6),
        (15, 13, 8, 7),
        (15, 13, 7, 4),
        (15, 13, 7, 3),
        (15, 12, 11, 5),
        (15, 12, 11, 3),
        (15, 12, 9, 4),
        (15, 12, 8, 6),
        (15, 12, 8, 3),
        (15, 12, 7, 4),
        (15, 12, 6, 5),
        (15, 11, 10, 5),
        (15, 11, 6, 5),
        ),
    16: (
        (16, 15, 13, 4),
        (16, 15, 12, 10),
        (16, 15, 12, 1),
        (16, 15, 10, 4),
        (16, 15, 9, 6),
        (16, 15, 9, 4),
        (16, 15, 7, 2),
        (16, 15, 4, 2),
        (16, 14, 13, 11),
        (16, 14, 13, 5),
        (16, 14, 12, 7),
        (16, 14, 11, 7),
        (16, 14, 9, 7),
        (16, 14, 9, 4),
        (16, 14, 8, 3),
        (16, 13, 12, 11),
        (16, 13, 12, 7),
        (16, 13, 11, 6),
        (16, 13, 9, 6),
        (16, 13, 6, 4),
        (16, 12, 9, 7),
        (16, 12, 9, 6),
        (16, 11, 10, 5),
        (16, 11, 9, 8),
        (16, 11, 9, 7),
        (16, 10, 9, 6),
        (16, 15, 14, 13, 12, 11),
        (16, 15, 14, 13, 11, 8),
        (16, 15, 14, 13, 11, 7),
        (16, 15, 14, 13, 10, 8),
        (16, 15, 14, 13, 10, 4),
        (16, 15, 14, 13, 9, 3),
        (16, 15, 14, 13, 6, 5),
        (16, 15, 14, 13, 6, 3),
        (16, 15, 14, 13, 6, 2),
        (16, 15, 14, 13, 4, 2),
        (16, 15, 14, 13, 4, 1),
        (16, 15, 14, 12, 10, 9),
        (16, 15, 14, 12, 10, 6),
        (16, 15, 14, 12, 9, 8),
        (16, 15, 14, 12, 9, 4),
        ),
    17: (
        (17, 14),
        (17, 12),
        (17, 11),
        (17, 16, 15, 14),
        (17, 16, 15, 7),
        (17, 16, 14, 9),
        (17, 16, 14, 6),
        (17, 16, 14, 4),
        (17, 16, 14, 1),
        (17, 16, 13, 12),
        (17, 16, 13, 5),
        (17, 16, 13, 2),
        (17, 16, 12, 10),
        (17, 16, 12, 8),
        (17, 16, 12, 4),
        (17, 16, 12, 1),
        (17, 16, 11, 7),
        (17, 16, 11, 1),
        (17, 16, 10, 9),
        (17, 16, 10, 8),
        (17, 16, 10, 5),
        (17, 16, 9, 5),
        (17, 16, 9, 4),
        (17, 16, 8, 6),
        (17, 16, 8, 3),
        (17, 16, 5, 4),
        (17, 16, 5, 3),
        (17, 16, 3, 2),
        (17, 15, 14, 12),
        (17, 15, 14, 10),
        (17, 15, 14, 9),
        (17, 15, 14, 8),
        (17, 15, 14, 6),
        (17, 15, 14, 5),
        (17, 15, 13, 11),
        (17, 15, 13, 7),
        (17, 15, 13, 5),
        (17, 15, 12, 9),
        (17, 15, 12, 2),
        (17, 15, 11, 10),
        (17, 15, 11, 8),
        ),
    18: (
        (18, 11),
        (18, 17, 16, 13),
        (18, 17, 16, 10),
        (18, 17, 16, 5),
        (18, 17, 15, 9),
        (18, 17, 14, 9),
        (18, 17, 13, 4),
        (18, 17, 13, 1),
        (18, 17, 10, 2),
        (18, 17, 8, 6),
        (18, 16, 15, 12),
        (18, 16, 13, 6),
        (18, 16, 13, 3),
        (18, 16, 11, 7),
        (18, 16, 11, 4),
        (18, 16, 10, 7),
        (18, 16, 9, 3),
        (18, 16, 8, 5),
        (18, 16, 7, 3),
        (18, 16, 5, 4),
        (18, 15, 14, 8),
        (18, 15, 13, 6),
        (18, 15, 13, 4),
        (18, 15, 11, 8),
        (18, 15, 11, 6),
        (18, 15, 9, 8),
        (18, 15, 9, 5),
        (18, 15, 9, 4),
        (18, 14, 13, 8),
        (18, 14, 13, 5),
        (18, 14, 12, 5),
        (18, 14, 11, 10),
        (18, 13, 12, 9),
        (18, 13, 11, 8),
        (18, 13, 11, 5),
        (18, 13, 10, 8),
        (18, 13, 9, 6),
        (18, 17, 16, 15, 14, 13),
        (18, 17, 16, 15, 13, 2),
        (18, 17, 16, 15, 12, 10),
        (18, 17, 16, 15, 12, 4),
        ),
    19: (
        (19, 18, 17, 14),
        (19, 18, 17, 13),
        (19, 18, 17, 5),
        (19, 18, 16, 10),
        (19, 18, 16, 5),
        (19, 18, 15, 13),
        (19, 18, 15, 12),
        (19, 18, 15, 10),
        (19, 18, 15, 3),
        (19, 18, 14, 13),
        (19, 18, 14, 1),
        (19, 18, 13, 11),
        (19, 18, 13, 8),
        (19, 18, 13, 3),
        (19, 18, 13, 1),
        (19, 18, 12, 10),
        (19, 18, 12, 3),
        (19, 18, 10, 6),
        (19, 18, 10, 2),
        (19, 18, 9, 6),
        (19, 18, 9, 4),
        (19, 18, 8, 7),
        (19, 18, 8, 2),
        (19, 18, 7, 5),
        (19, 18, 7, 3),
        (19, 18, 5, 2),
        (19, 17, 16, 6),
        (19, 17, 15, 8),
        (19, 17, 14, 10),
        (19, 17, 14, 3),
        (19, 17, 13, 8),
        (19, 17, 13, 5),
        (19, 17, 12, 11),
        (19, 17, 12, 10),
        (19, 17, 12, 4),
        (19, 17, 11, 6),
        (19, 17, 10, 6),
        (19, 17, 10, 3),
        (19, 17, 7, 3),
        (19, 17, 6, 5),
        (19, 17, 5, 4),
        ),
    20: (
        (20, 17),
        (20, 19, 16, 14),
        (20, 19, 16, 2),
        (20, 19, 15, 11),
        (20, 19, 15, 10),
        (20, 19, 15, 4),
        (20, 19, 14, 9),
        (20, 19, 14, 4),
        (20, 19, 12, 8),
        (20, 19, 11, 7),
        (20, 19, 9, 8),
        (20, 19, 9, 2),
        (20, 19, 8, 4),
        (20, 19, 7, 6),
        (20, 19, 4, 3),
        (20, 18, 15, 14),
        (20, 18, 13, 10),
        (20, 18, 13, 7),
        (20, 18, 11, 10),
        (20, 18, 11, 8),
        (20, 18, 11, 5),
        (20, 18, 9, 3),
        (20, 17, 16, 9),
        (20, 17, 15, 14),
        (20, 17, 15, 11),
        (20, 17, 15, 9),
        (20, 17, 14, 8),
        (20, 17, 14, 3),
        (20, 17, 13, 4),
        (20, 17, 12, 10),
        (20, 17, 11, 9),
        (20, 17, 9, 8),
        (20, 17, 9, 7),
        (20, 17, 9, 4),
        (20, 17, 6, 4),
        (20, 16, 15, 11),
        (20, 16, 15, 5),
        (20, 16, 13, 9),
        (20, 16, 13, 5),
        (20, 16, 12, 7),
        (20, 16, 10, 9),
        ),
    21: (
        (21, 19),
        (21, 20, 19, 16),
        (21, 20, 19, 13),
        (21, 20, 19, 7),
        (21, 20, 19, 5),
        (21, 20, 19, 4),
        (21, 20, 19, 2),
        (21, 20, 18, 6),
        (21, 20, 17, 14),
        (21, 20, 17, 10),
        (21, 20, 17, 7),
        (21, 20, 17, 3),
        (21, 20, 17, 2),
        (21, 20, 16, 11),
        (21, 20, 16, 2),
        (21, 20, 14, 10),
        (21, 20, 14, 8),
        (21, 20, 13, 7),
        (21, 20, 11, 9),
        (21, 20, 9, 3),
        (21, 20, 8, 3),
        (21, 20, 7, 5),
        (21, 20, 6, 5),
        (21, 20, 5, 2),
        (21, 19, 18, 10),
        (21, 19, 17, 11),
        (21, 19, 17, 10),
        (21, 19, 17, 4),
        (21, 19, 16, 15),
        (21, 19, 16, 12),
        (21, 19, 16, 8),
        (21, 19, 16, 5),
        (21, 19, 15, 9),
        (21, 19, 15, 4),
        (21, 19, 14, 13),
        (21, 19, 14, 7),
        (21, 19, 13, 6),
        (21, 19, 12, 6),
        (21, 19, 11, 8),
        (21, 19, 11, 4),
        (21, 19, 10, 5),
        ),
    22: (
        (22, 21),
        (22, 21, 20, 11),
        (22, 21, 20, 7),
        (22, 21, 19, 10),
        (22, 21, 19, 2),
        (22, 21, 18, 12),
        (22, 21, 17, 13),
        (22, 21, 17, 8),
        (22, 21, 16, 15),
        (22, 21, 15, 6),
        (22, 21, 14, 11),
        (22, 21, 13, 1),
        (22, 21, 12, 9),
        (22, 21, 10, 9),
        (22, 21, 10, 6),
        (22, 21, 10, 5),
        (22, 21, 7, 2),
        (22, 21, 5, 4),
        (22, 20, 19, 8),
        (22, 20, 19, 3),
        (22, 20, 16, 3),
        (22, 20, 14, 13),
        (22, 20, 13, 9),
        (22, 20, 13, 8),
        (22, 20, 13, 6),
        (22, 20, 11, 10),
        (22, 20, 11, 9),
        (22, 20, 11, 4),
        (22, 20, 11, 3),
        (22, 19, 18, 17),
        (22, 19, 17, 14),
        (22, 19, 17, 11),
        (22, 19, 17, 9),
        (22, 19, 16, 6),
        (22, 19, 16, 4),
        (22, 19, 15, 12),
        (22, 19, 15, 10),
        (22, 19, 15, 5),
        (22, 19, 14, 5),
        (22, 19, 12, 9),
        (22, 19, 6, 5),
        ),
    23: (
        (23, 18),
        (23, 14),
        (23, 22, 21, 16),
        (23, 22, 21, 8),
        (23, 22, 21, 2),
        (23, 22, 21, 1),
        (23, 22, 20, 18),
        (23, 22, 20, 16),
        (23, 22, 20, 4),
        (23, 22, 20, 3),
        (23, 22, 19, 18),
        (23, 22, 19, 8),
        (23, 22, 19, 3),
        (23, 22, 18, 16),
        (23, 22, 18, 11),
        (23, 22, 18, 8),
        (23, 22, 18, 5),
        (23, 22, 18, 4),
        (23, 22, 17, 10),
        (23, 22, 17, 6),
        (23, 22, 16, 15),
        (23, 22, 16, 8),
        (23, 22, 16, 6),
        (23, 22, 15, 14),
        (23, 22, 15, 1),
        (23, 22, 14, 12),
        (23, 22, 14, 10),
        (23, 22, 14, 7),
        (23, 22, 13, 6),
        (23, 22, 12, 10),
        (23, 22, 12, 8),
        (23, 22, 12, 4),
        (23, 22, 11, 10),
        (23, 22, 10, 5),
        (23, 22, 8, 7),
        (23, 22, 8, 4),
        (23, 22, 6, 5),
        (23, 22, 5, 4),
        (23, 22, 4, 2),
        (23, 21, 20, 18),
        (23, 21, 20, 17),
        ),
    24: (
        (24, 23, 22, 17),
        (24, 23, 22, 7),
        (24, 23, 21, 20),
        (24, 23, 21, 11),
        (24, 23, 21, 8),
        (24, 23, 19, 2),
        (24, 23, 18, 14),
        (24, 23, 16, 13),
        (24, 23, 14, 1),
        (24, 23, 10, 8),
        (24, 23, 9, 3),
        (24, 23, 5, 4),
        (24, 22, 21, 12),
        (24, 22, 21, 9),
        (24, 22, 20, 7),
        (24, 22, 19, 16),
        (24, 22, 19, 15),
        (24, 22, 19, 13),
        (24, 22, 19, 8),
        (24, 22, 19, 6),
        (24, 22, 18, 13),
        (24, 22, 17, 8),
        (24, 22, 16, 7),
        (24, 22, 15, 4),
        (24, 22, 15, 3),
        (24, 22, 14, 13),
        (24, 22, 9, 3),
        (24, 22, 8, 7),
        (24, 22, 7, 6),
        (24, 21, 20, 14),
        (24, 21, 18, 14),
        (24, 21, 18, 5),
        (24, 21, 11, 5),
        (24, 20, 19, 17),
        (24, 20, 19, 5),
        (24, 20, 18, 15),
        (24, 20, 11, 9),
        (24, 20, 11, 5),
        (24, 19, 14, 11),
        (24, 19, 13, 11),
        (24, 19, 9, 6),
        ),
    25: (
        (25, 22),
        (25, 18),
        (25, 24, 23, 22),
        (25, 24, 23, 2),
        (25, 24, 22, 15),
        (25, 24, 22, 13),
        (25, 24, 22, 8),
        (25, 24, 22, 5),
        (25, 24, 22, 3),
        (25, 24, 21, 18),
        (25, 24, 21, 9),
        (25, 24, 20, 16),
        (25, 24, 20, 13),
        (25, 24, 20, 11),
        (25, 24, 20, 8),
        (25, 24, 20, 4),
        (25, 24, 19, 11),
        (25, 24, 19, 8),
        (25, 24, 19, 1),
        (25, 24, 18, 4),
        (25, 24, 18, 3),
        (25, 24, 17, 8),
        (25, 24, 17, 4),
        (25, 24, 17, 2),
        (25, 24, 17, 1),
        (25, 24, 16, 13),
        (25, 24, 16, 9),
        (25, 24, 16, 7),
        (25, 24, 16, 1),
        (25, 24, 15, 12),
        (25, 24, 15, 3),
        (25, 24, 13, 12),
        (25, 24, 13, 11),
        (25, 24, 13, 6),
        (25, 24, 13, 5),
        (25, 24, 13, 4),
        (25, 24, 13, 1),
        (25, 24, 12, 9),
        (25, 24, 11, 10),
        (25, 24, 11, 7),
        (25, 24, 11, 5),
        ),
    26: (
        (26, 25, 24, 20),
        (26, 25, 24, 8),
        (26, 25, 24, 7),
        (26, 25, 22, 17),
        (26, 25, 22, 14),
        (26, 25, 22, 3),
        (26, 25, 22, 2),
        (26, 25, 21, 1),
        (26, 25, 20, 15),
        (26, 25, 19, 18),
        (26, 25, 19, 3),
        (26, 25, 18, 13),
        (26, 25, 18, 2),
        (26, 25, 17, 9),
        (26, 25, 17, 7),
        (26, 25, 16, 5),
        (26, 25, 15, 13),
        (26, 25, 14, 3),
        (26, 25, 13, 3),
        (26, 25, 13, 2),
        (26, 25, 12, 11),
        (26, 25, 12, 7),
        (26, 25, 11, 9),
        (26, 25, 11, 7),
        (26, 25, 10, 8),
        (26, 25, 10, 6),
        (26, 25, 9, 4),
        (26, 25, 8, 6),
        (26, 25, 7, 3),
        (26, 24, 23, 20),
        (26, 24, 23, 10),
        (26, 24, 22, 7),
        (26, 24, 21, 5),
        (26, 24, 20, 17),
        (26, 24, 20, 9),
        (26, 24, 19, 17),
        (26, 24, 18, 5),
        (26, 24, 17, 9),
        (26, 24, 17, 7),
        (26, 24, 17, 5),
        (26, 24, 16, 13),
        ),
    27: (
        (27, 26, 25, 22),
        (27, 26, 25, 17),
        (27, 26, 25, 11),
        (27, 26, 24, 11),
        (27, 26, 23, 7),
        (27, 26, 22, 10),
        (27, 26, 22, 2),
        (27, 26, 22, 1),
        (27, 26, 21, 11),
        (27, 26, 20, 19),
        (27, 26, 20, 9),
        (27, 26, 18, 11),
        (27, 26, 17, 9),
        (27, 26, 17, 1),
        (27, 26, 16, 14),
        (27, 26, 16, 7),
        (27, 26, 16, 2),
        (27, 26, 15, 2),
        (27, 26, 14, 9),
        (27, 26, 14, 7),
        (27, 26, 13, 2),
        (27, 26, 12, 6),
        (27, 26, 10, 5),
        (27, 26, 6, 3),
        (27, 26, 3, 2),
        (27, 25, 24, 15),
        (27, 25, 23, 10),
        (27, 25, 23, 7),
        (27, 25, 22, 18),
        (27, 25, 22, 14),
        (27, 25, 22, 3),
        (27, 25, 21, 15),
        (27, 25, 21, 9),
        (27, 25, 21, 4),
        (27, 25, 20, 17),
        (27, 25, 20, 11),
        (27, 25, 20, 8),
        (27, 25, 20, 4),
        (27, 25, 19, 18),
        (27, 25, 19, 8),
        (27, 25, 19, 3),
        ),
    28: (
        (28, 25),
        (28, 19),
        (28, 15),
        (28, 27, 26, 16),
        (28, 27, 26, 11),
        (28, 27, 25, 18),
        (28, 27, 25, 6),
        (28, 27, 24, 22),
        (28, 27, 24, 7),
        (28, 27, 23, 19),
        (28, 27, 23, 4),
        (28, 27, 21, 13),
        (28, 27, 21, 7),
        (28, 27, 20, 14),
        (28, 27, 20, 11),
        (28, 27, 17, 1),
        (28, 27, 16, 5),
        (28, 27, 16, 3),
        (28, 27, 15, 7),
        (28, 27, 15, 3),
        (28, 27, 14, 5),
        (28, 27, 13, 8),
        (28, 27, 13, 4),
        (28, 27, 8, 7),
        (28, 27, 5, 4),
        (28, 26, 23, 15),
        (28, 26, 23, 13),
        (28, 26, 23, 9),
        (28, 26, 21, 14),
        (28, 26, 21, 5),
        (28, 26, 20, 13),
        (28, 26, 20, 7),
        (28, 26, 19, 6),
        (28, 26, 15, 11),
        (28, 26, 10, 5),
        (28, 26, 10, 3),
        (28, 26, 6, 5),
        (28, 25, 24, 11),
        (28, 25, 24, 9),
        (28, 25, 23, 18),
        (28, 25, 22, 12),
        ),
    29: (
        (29, 27),
        (29, 28, 27, 25),
        (29, 28, 27, 20),
        (29, 28, 27, 8),
        (29, 28, 27, 4),
        (29, 28, 26, 10),
        (29, 28, 25, 6),
        (29, 28, 24, 19),
        (29, 28, 24, 2),
        (29, 28, 23, 22),
        (29, 28, 23, 19),
        (29, 28, 23, 15),
        (29, 28, 21, 11),
        (29, 28, 20, 10),
        (29, 28, 20, 7),
        (29, 28, 19, 15),
        (29, 28, 17, 10),
        (29, 28, 17, 3),
        (29, 28, 15, 8),
        (29, 28, 15, 5),
        (29, 28, 14, 4),
        (29, 28, 13, 6),
        (29, 28, 12, 10),
        (29, 28, 11, 4),
        (29, 28, 10, 2),
        (29, 28, 8, 6),
        (29, 27, 26, 25),
        (29, 27, 26, 22),
        (29, 27, 26, 19),
        (29, 27, 26, 17),
        (29, 27, 26, 15),
        (29, 27, 26, 11),
        (29, 27, 25, 20),
        (29, 27, 25, 14),
        (29, 27, 25, 13),
        (29, 27, 25, 7),
        (29, 27, 24, 15),
        (29, 27, 24, 11),
        (29, 27, 24, 9),
        (29, 27, 24, 8),
        (29, 27, 24, 3),
        ),
    30: (
        (30, 29, 28, 7),
        (30, 29, 26, 24),
        (30, 29, 26, 22),
        (30, 29, 26, 4),
        (30, 29, 24, 17),
        (30, 29, 24, 12),
        (30, 29, 23, 20),
        (30, 29, 23, 18),
        (30, 29, 23, 13),
        (30, 29, 23, 8),
        (30, 29, 22, 7),
        (30, 29, 19, 17),
        (30, 29, 19, 10),
        (30, 29, 19, 8),
        (30, 29, 18, 9),
        (30, 29, 18, 3),
        (30, 29, 17, 7),
        (30, 29, 17, 5),
        (30, 29, 16, 4),
        (30, 29, 15, 14),
        (30, 29, 14, 2),
        (30, 29, 12, 3),
        (30, 29, 10, 4),
        (30, 29, 9, 6),
        (30, 29, 8, 5),
        (30, 28, 27, 6),
        (30, 28, 26, 7),
        (30, 28, 23, 17),
        (30, 28, 23, 11),
        (30, 28, 23, 10),
        (30, 28, 23, 4),
        (30, 28, 22, 17),
        (30, 28, 21, 12),
        (30, 28, 21, 7),
        (30, 28, 20, 19),
        (30, 28, 20, 17),
        (30, 28, 19, 7),
        (30, 28, 18, 7),
        (30, 28, 16, 9),
        (30, 28, 15, 12),
        (30, 28, 14, 5),
        ),
    31: (
        (31, 28),
        (31, 25),
        (31, 24),
        (31, 18),
        (31, 30, 29, 28),
        (31, 30, 29, 25),
        (31, 30, 29, 20),
        (31, 30, 29, 19),
        (31, 30, 29, 5),
        (31, 30, 28, 24),
        (31, 30, 28, 22),
        (31, 30, 28, 18),
        (31, 30, 28, 16),
        (31, 30, 28, 13),
        (31, 30, 28, 10),
        (31, 30, 28, 8),
        (31, 30, 28, 3),
        (31, 30, 27, 16),
        (31, 30, 26, 22),
        (31, 30, 26, 8),
        (31, 30, 25, 16),
        (31, 30, 25, 9),
        (31, 30, 25, 7),
        (31, 30, 25, 5),
        (31, 30, 24, 19),
        (31, 30, 24, 16),
        (31, 30, 24, 12),
        (31, 30, 23, 20),
        (31, 30, 23, 19),
        (31, 30, 23, 5),
        (31, 30, 22, 21),
        (31, 30, 22, 20),
        (31, 30, 22, 12),
        (31, 30, 22, 4),
        (31, 30, 21, 18),
        (31, 30, 21, 10),
        (31, 30, 20, 11),
        (31, 30, 20, 8),
        (31, 30, 20, 7),
        (31, 30, 20, 1),
        (31, 30, 19, 17),
        ),
    32: (
        (32, 31, 30, 10),
        (32, 31, 29, 1),
        (32, 31, 26, 18),
        (32, 31, 26, 9),
        (32, 31, 26, 7),
        (32, 31, 23, 10),
        (32, 31, 22, 17),
        (32, 31, 21, 16),
        (32, 31, 21, 5),
        (32, 31, 18, 10),
        (32, 31, 16, 2),
        (32, 31, 15, 10),
        (32, 31, 14, 4),
        (32, 31, 13, 8),
        (32, 31, 9, 7),
        (32, 31, 5, 4),
        (32, 30, 29, 23),
        (32, 30, 29, 20),
        (32, 30, 29, 16),
        (32, 30, 29, 15),
        (32, 30, 27, 24),
        (32, 30, 27, 21),
        (32, 30, 27, 12),
        (32, 30, 27, 8),
        (32, 30, 26, 25),
        (32, 30, 26, 13),
        (32, 30, 25, 16),
        (32, 30, 23, 16),
        (32, 30, 23, 14),
        (32, 30, 23, 4),
        (32, 30, 21, 14),
        (32, 30, 19, 8),
        (32, 30, 19, 4),
        (32, 30, 17, 3),
        (32, 30, 15, 6),
        (32, 30, 11, 8),
        (32, 30, 11, 5),
        (32, 30, 8, 3),
        (32, 30, 7, 4),
        (32, 29, 28, 19),
        (32, 29, 27, 23),
        ),
    }

if __name__ == '__main__':
    pass
